// An associative data structure holding entries (each consisting of a key and a value) within arrays;
// the sizes of the arrays are increased on demand up to the maximal array size;
// null can be used as key as well as value;
// all methods except 'size' use linear search to find entries causing slow execution in all cases
public class SimpleAssoc implements Assoc {
    private int top;
    private String[] ks = new String[8];
    private String[] vs = new String[8];

    private int find(String s, String[] a) {
        int i = 0;
        while (i < top &&
                !(s == null ? s == a[i] : s.equals(a[i])))
            i++;
        return i;
    }


    // inserts an entry with key k and value v if there was no entry with key k before;
    // substitutes v for the value of an entry with key k if there already was such an entry;
    // returns the old value of this entry or null (if there was no such entry before or the old value was null)
    public String put(String k, String v) {
        int i = find(k, ks);
        if (i == top && ++top == ks.length) {
            String[] nks = new String[top << 1];
            String[] nvs = new String[top << 1];
            for (int j = 0; j < i; j++) {
                nks[j] = ks[j];
                nvs[j] = vs[j];
            }
            ks = nks;
            vs = nvs;
        }
        ks[i] = k;
        String old = vs[i];
        vs[i] = v;
        return old;
    }

    // removes an entry with a key equal to k if there is such an entry;
    // returns the old value of this entry or null (if there was no such entry or the old value was null)
    public String remove(String k) {
        int i = find(k, ks);
        String old = vs[i];
        if (i < top) {
            ks[i] = ks[--top];
            ks[top] = null;
            vs[i] = vs[top];
            vs[top] = null;
        }
        return old;
    }

    // returns the value of an entry where the key equals k;
    // returns null if there is no such entry (or if the value of the entry is null)
    public String get(String k) {
        return vs[find(k, ks)];
    }

    // returns true if there is an entry with a key equal to k
    public boolean containsKey(String k) {
        return find(k, ks) < top;
    }

    // returns true if there is an entry with a value equal to v
    public boolean containsValue(String v) {
        return find(v, vs) < top;
    }

    // returns the current number of entries
    public int size() {
        return top;
    }


    // returns a readable representation of this data structure containing each entry
    public String toString() {
        if (top <= 0) {
            return "{}";
        }
        String s = "(" + ks[0] + ", " + vs[0] + ")";
        for (int i = 1; i < top; i++) {
            s += ", (" + ks[i] + ", " + vs[i] + ")";
        }
        return "{" + s + "}";
    }

}
