// An associative data structure holding entries (each consisting of a key and a value) in an unbalanced binary search tree;
// the number of entries is limited only by the available memory size;
// null can be used as key as well as value;
// 'containsValue' may need to traverse the whole tree when searching for entries (slow);
// all other methods use tree search (usually fast, but can be slow and may fail if data are distributed in a bad way)
public class TreeAssoc implements Assoc {
    private TANode root;

    // inserts an entry with key k and value v if there was no entry with key k before;
    // substitutes v for the value of an entry with key k if there already was such an entry;
    // returns the old value of this entry or null (if there was no such entry before or the old value was null)
    public String put(String k, String v) {
        if (root != null) {
            return root.put(k, v);
        }
        root = new TANode(k, v);
        return null;
    }

    // returns the value of an entry where the key equals k;
    // returns null if there is no such entry (or if the value of the entry is null)
    public String get(String k) {
        if (root == null) {
            return null;
        }
        TANode node = root.find(k);
        return node == null ? null : node.value();
    }

    // returns true if there is an entry with a key equal to k
    public boolean containsKey(String k) {
        return root != null && root.find(k) != null;
    }

    // returns true if there is an entry with a value equal to v
    public boolean containsValue(String v) {
        return root != null && root.hasValue(v);
    }

    // returns a readable representation of this data structure containing each entry;
    // entries in ascending order of their keys
    public String toString() {
        return root == null ? "{}" : "{" + root + "}";
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null || getClass() != obj.getClass()) {
            return false;
        }
        TreeAssoc that = (TreeAssoc) obj;
        if (this.root == that.root) {
            return true;
        }
        if (this.root == null || that.root == null) {
            return false;
        }
        TANode nil = new TANode(null, null);
        return root.equalsList(that.root.asList(nil)) == nil;
    }

    @Override
    public int hashCode() {
        int i = -450912453;
        if (root != null) {
            i += root.hashCodeSum();
        }
        return i;
    }
}
